! ==============================================================================
! Simple Compute Pi program using SPMD pattern. 
! Compile with:
!   gfortran -fopenmp ompute_pi_f90_spmd-template.f90
!   ifort -openmp compute_pi_f90_spmd-template.f90
! ==============================================================================

program compute_pi

   iADD_OMP_DIRECTIVE_HERE        ! Include the module to anable Runtime library.   

   implicit none
   integer, parameter :: nb_steps = 20000000          ! Number of steps.
   real(8), parameter :: step     = 1.0d0 / nb_steps  ! Width of the small rectangle.
   real(8) :: pi, x
   real(8) :: start_time, end_time, elapsed_time      ! Time variables to compute the cpu time used.
   integer :: i, j
   real(8) :: tot_sum
   integer :: numthreads, id

   integer, parameter :: MAX_THREADS = 4

   real(8) :: sum(MAX_THREADS)

   do j = 1, MAX_THREADS

      ADD_OMP_DIRECTIVE_HERE                    ! Set number of threads to j

      start_time = ADD_OMP_DIRECTIVES_HERE      ! Get the start time.

      ! Start the parallel region here.
        
      ADD_OMP_DIRECTIVES_HERE                   ! Create threads. 

      id = ADD_OMP_DIRECTIVES_HERE              ! Get the thread rank 
                                                   
      numthreads = ADD_OMP_DIRECTIVES_HERE      ! Get number of threads.

      sum(id) = 0.0d0                           ! Initialize the sum variable.  

      do i = 1 + id, nb_steps, numthreads       ! Loop over the number of steps. 
         x = (i + 0.5) * step                   ! Compute the position of the rectangle.
         sum(id) = sum(id) + 1.0/(1.0 + x * x)  ! Update the sum by adding the surface of the current rectangle.
      end do

      ! close the parallel region
        
      ADD_OMP_DIRECTIVES_HERE 

      tot_sum = 0.0
      do i = 1, j
         tot_sum = tot_sum + sum(i)
      end do
 
      pi = 4.0 * tot_sum * step                 ! Multiply by the factor: 4.0 * step
 
      end_time = ADD_OMP_DIRECTIVES_HERE        ! Get the end time.

      elapsed_time = end_time - start_time      ! Compute the elapsed time in seconds.

      ! Print out the results: pi value, number of steps, elapsed time

      write(*,fmt="(a,f15.12,a,I15,a,f15.12,a)") &
            & "The value of pi is [ ", pi, "; Computed using [ ", nb_steps, &
            & "] steps in [ ", elapsed_time, "]  seconds"

   end do

end program compute_pi

! ==============================================================================
! End of the program Compute Pi.
! ==============================================================================
