/*
 * ==============================================================================
 * Simple Program to Compute PI: OpenMP Version.
 * Compile with:
 *    gcc -fopenmp compute_pi_c_omp_atomic-template.c
 *    icc -openmp compute_pi_c_omp_atomic-template.c
 * ==============================================================================
*/

#include <omp.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#define nb_steps 20000000                    /* Number of steps        */
#define step (1.0 / (double) nb_steps)       /* Width of the small rectangle */
#define MAX_THREADS 4

double pi;

int main() {
  double start_time, end_time, elapsed_time;   /* Time variables to compute the cpu time used */
  double pi;
  int  i, j;
  double total_sum = 0.0; 
  
  for (j = 1; j <= MAX_THREADS; j++) {

      omp_set_num_threads(j);

      total_sum = 0.0;

      start_time = omp_get_wtime();                                     /* Get the start time */                         

      ADD_OMP_DIRECTIVE_HERE             /* Start a parallel region and add private variables */ 
      {
         int id = omp_get_thread_num();
         int numthreads = omp_get_num_threads();
         double x;
         double partial_sum = 0.0;                            /* Initialize the sum variable. */ 

         ADD_OMP_DIRECTIVE_HERE                               /* Use single construct */ 
            printf("The Number of Threads = %d \n", numthreads);

            for (i = id; i < nb_steps; i+=numthreads) {        /* Loop over the number of steps. */
               x = (i + 0.5) * step;                           /* Compute the position of the rectangle. */
	       partial_sum = partial_sum + 1.0/(1.0 + x * x);  /* Update the sum by adding the surface of current rectangle. */
            }
         ADD_OMP_DIRECTIVE_HERE                                /* Use atomic construct */
	    total_sum += partial_sum;
      }

      pi = 4.0 * total_sum * step;                     /* Multiply by the factor: 4.0 * step */

      end_time = omp_get_wtime();                      /* Get the end time */

      /* Compute the elapsed time in seconds */

      elapsed_time = ((double)(end_time - start_time));

      /* Print out the results: pi value, number of steps, elapsed time */

      printf("The value of pi is [ %15.12lf ]; Computed using [ %ld ] steps in [ %15.12lf ] seconds\n",pi,nb_steps,elapsed_time);

  }

}

// End of the program Compute Pi.
