!==============================================================================
! Simple Hello World program: OpenMP Version.
!
! Compile with:
! - gfortran -fopenmp helloworld_f90_omp.f90 -o helloworld_f90_omp
! - ifort -openmp helloworld_f90_omp.f90 -o helloworld_f90_omp 
!==============================================================================

program Hello_World

   use omp_lib

   implicit none
   integer, parameter :: NUM_THREADS = 4 
   integer :: ID
   integer :: nthr
   integer :: nthreads

   real(8) :: start_time, end_time, elapsed_time; ! Time variables to compute the cpu time used.

   write(*,*) 
   write(*,fmt="(a)") "Start the program."
   write(*,*) " "

   ! Set number of threads to NUM_THREADS

   call omp_set_num_threads(NUM_THREADS)

   ! To know how many threads are available outside the parallel region.

   nthr = omp_get_num_threads();

   write(*,fmt="(a,I2)") "I am out of the parallel region and the number of threads is: ", nthr

   write(*,*)
   write(*,fmt="(a)") "Start of the parallel region."
   write(*,*) " "

   ! Store the time before the parallel region. 

   start_time = omp_get_wtime()

   ! Add !$omp parallel to create threads

!$omp parallel default(none) private(ID) shared (nthreads) 

   ID = omp_get_thread_num()          ! To get the thread number or rank.

   nthreads = omp_get_num_threads()   ! To get the number of threads in the parallel region.

   write(*,fmt="(a,I2,a,I2,a)") "Hello World!, My ID is equal to [ ", ID, " ] - The total of threads is: [", nthreads, " ]"
 
!$omp end parallel

   write(*,*) " "
   write(*,fmt="(a)") "End of the parallel region."
   write(*,*) " "

   ! Store the time after the parallel region. 

   end_time = omp_get_wtime()

   ! Compute time spend in the parallel region. 

   elapsed_time = end_time - start_time

   ! Print out the time spend in the parallel region. */

   write(*,fmt="(a,f12.8,a)") "The time spend in the parallel region is: ",elapsed_time, "s"

   ! To know how many threads are available outside the parallel region.

   nthr = omp_get_num_threads()

   write(*,fmt="(a,I2)") "I am out of the parallel region and the number of threads is: ", nthr
   write(*,*) " "
   write(*,fmt="(a)") "End of the program."
   write(*,*) " "

end program Hello_World

!==============================================================================
! End of the program Hello World.
!==============================================================================
